<?php
/**
 * User: nester_a
 * Date: 31.10.13
 */

namespace LMS\Libraries\Cache;

class Helper
{
    private $cacheDriver;
    private $config = null;
    private static $instance = null;

    private function __construct($cacheDriver = false, $options=false)
    {
        if(!$cacheDriver){
            jimport('joomla.cache.storage.file');
            $config = !is_null($this->config) ? $this->config : \JFactory::getConfig();
            $options = $options ? $options : array(
                'cachebase' => $config->get('tmp_path')
            );
            $cacheDriver = new \JCacheStorageFile($options);
        }
        $this->cacheDriver = $cacheDriver;
    }

    public static function getInstance($cacheDriver = false, $options=false)
    {
        if(is_null(self::$instance)){
            self::$instance = new Helper($cacheDriver, $options);
        }
        return self::$instance;
    }

    public function get($id, $group='common')
    {
        return $this->cacheDriver->get($id, $group);
    }

    public function set($id, $data, $group='common')
    {
        $this->cacheDriver->store($id, $group, $data);
    }

    public function setCacheDriver($cacheDriver)
    {
        $this->cacheDriver = $cacheDriver;
    }

    public function setConfig($config)
    {
        $this->config = $config;
    }

    public function startTemplateCache($id, $group='template')
    {
        $additionalJS = $this->cacheDriver->get($id, 'add_js');
        if($additionalJS){
            $this->config->set('jlms_aditional_js_code', $this->config->get('jlms_aditional_js_code').$additionalJS);
        }
        $headerContent = $this->cacheDriver->get($id, $group);
        if($headerContent){
            echo $headerContent;
            return true;
        }
        ob_start();
        return false;
    }

    public function endTemplateCache($id, $group='template')
    {
        $headerContent = ob_get_contents();
        $this->cacheDriver->store($id, $group, $headerContent);
        $this->cacheDriver->store($id, 'add_js', $this->config->get('jlms_aditional_js_code',''));
        ob_end_clean();
        echo $headerContent;
    }

    public function disableCaching($disabledGroups=array())
    {
        if(!($this->cacheDriver instanceof DisableCacheDecorator)){
            $this->cacheDriver = new DisableCacheDecorator($this->cacheDriver, $disabledGroups) ;
        }
        $this->cacheDriver->addGroups($disabledGroups);
    }

    public function enableCaching($disabledGroups=array())
    {
        if(!($this->cacheDriver instanceof JCacheStorage)){
            if(empty($disabledGroups)){
                $this->cacheDriver = $this->cacheDriver->getDecoratedObject();
            }else{
                $this->cacheDriver->removeGroups($disabledGroups);
            }
        }
    }

}

class DisableCacheDecorator extends \JCacheStorageFile
{
    private $decoratedObject;
    private $disabledGroups;

    public function __construct($decoratedObject, $disabledGroups = array())
    {
        $this->decoratedObject = $decoratedObject;
        $this->disabledGroups = $disabledGroups;
    }

    public function store($id, $group, $data)
    {
        $disabledGroups = $this->disabledGroups;
        if(empty($disabledGroups)){
            return false;
        }
        if(!in_array($group, $disabledGroups)){
            return $this->decoratedObject->store($id, $group, $data);
        }
        return false;
    }

    public function get($id, $group, $checkTime = true)
    {
        $disabledGroups = $this->disabledGroups;
        if(empty($disabledGroups)){
            return false;
        }
        if(!in_array($group, $disabledGroups)){
            return $this->decoratedObject->get($id, $group);
        }
        return false;
    }

    public function getDecoratedObject()
    {
        return $this->decoratedObject;
    }

    public function addGroups($disabledGroups)
    {
        $this->disabledGroups = array_merge($this->disabledGroups, $disabledGroups);
    }

    public function removeGroups($disabledGroups)
    {
        $this->disabledGroups = array_diff($this->disabledGroups, $disabledGroups);
    }

}