<?php
namespace LMS\Tools\Announcements\Repositories;

use LMS\Libraries\Base;
use LMS\Tools\Announcements\Tables;
use LMS\Libraries\ServiceLocator;

class Announcements extends Base\Repository
{
    private $calendar_query = null;

	public function __construct($options = array())
	{
		parent::__construct($options);
		$this->table = new Tables\Announcements($this->db);
        $this->acl = ServiceLocator::get('acl');
	}

    public function getCalendarQuery()
    {
        return $this->calendar_query;
    }

    protected function prepareFilter($query, &$filterOptions)
    {

        $alias = $this->table->getTableAlias();

        $conditions = array();

        if(isset($filterOptions['announcement_filter']) && $filterOptions['announcement_filter']){
            switch ($filterOptions['announcement_filter']) {
                case "all":
                    break;
                case "started":
                    $filterOptions['announcements.started']= 1;
                    break;
                case "upcoming":
                    $filterOptions['announcements.upcoming']= 1;
                    break;
                case "finished":
                    $filterOptions['announcements.finished']= 1;
                    break;
            }
        }

        if(isset($filterOptions['announcements.course_id']) && $filterOptions['announcements.course_id']){
            $conditions[] = $alias.".course_id = ".$this->db->quote($filterOptions['announcements.course_id']);
        }

        if(isset($filterOptions['announcements.featured']) && $filterOptions['announcements.featured']){
            $conditions[] = $alias.".featured = ".$this->db->quote($filterOptions['announcements.featured']);
        }else
        {
           // $conditions[] = $alias.".featured = 0";
        }

        if(isset($filterOptions['announcements.started']) && $filterOptions['announcements.started']){
            $conditions[] = $alias.".start_date < NOW() AND ".$alias.".end_date > NOW()";
        }

        if(isset($filterOptions['announcements.finished']) && $filterOptions['announcements.finished']){
            $conditions[] = $alias.".start_date < NOW() AND ".$alias.".end_date < NOW()";
        }

        if(isset($filterOptions['announcements.upcoming']) && $filterOptions['announcements.upcoming']){
            $conditions[] = $alias.".start_date > NOW() AND ".$alias.".end_date > NOW()";
        }

        if (!$this->acl->CheckPermissions('announce', 'manage'))
        {
            $uid = (int)\JFactory::getUser()->id;
            $course_id = (int) $filterOptions['announcements.course_id'];
            $enroll_period = (int)JLMS_getEnrolPeriod( $uid, $course_id);
            $conditions[] = "(".$alias.".show_period <= ".$enroll_period." OR ".$alias.".show_period=0)";
            $conditions[] = "(".$alias.".start_publish_date <= NOW() AND ".$alias.".end_publish_date >= NOW() OR (".$alias.".start_publish_date = '0000-00-00 00:00:00' AND ".$alias.".end_publish_date = '0000-00-00 00:00:00'))";
        }


        //$query->select($alias.'.*');
        //$query->select('DATEDIFF(NOW(), `start_date`) AS `dt` ');
        if (sizeof($conditions)) $query->where($conditions);


        //$query->order(array($alias.'.`start_date` desc'));
        $query->order('ABS(DATEDIFF( NOW(), `start_date`)) ASC');
        //echo 'SMT DEBUG: <pre>'; print_R($filterOptions); echo '</pre>';//die;
        //echo 'SMT DEBUG: <pre>'; print_R($query->__toString()); echo '</pre>';//die;
        //$this->calendar_query = $query;
        return parent::prepareFilter($query, $filterOptions);
    }

}