<?php
/**
 * @package     Joomla.Site
 * @subpackage  com_joomla_lms
 *
 */

namespace LMS\Tools\Conference\Controllers;

defined('_JEXEC') or die;

use LMS\Libraries\Base;
use LMS\Tools\Conference\Response;
use LMS\Libraries\ServiceLocator;
use LMS\Libraries\Date;
use LMS\Libraries\Exception;

/**
 * @package     Joomla.Site
 * @subpackage  com_joomla_lms
 * @since       3.1
 */
class Conference extends Base\Controller
{

	public function __construct($config=array())
	{
		parent::__construct($config);
		$this->_init();
		$this->registerDefaultTask('custom');
		$this->input->set('view', 'conference');

		$this->responseHandler = new Response\Handler($this->container);

		if($this->container['config']->get('conference_enable') == 0){
			throw new Exception\Access\Forbidden(_JLMS_ERROR_ACCESS_FORBIDDEN);
		}

	}


	/**
	 * Method to display a view.
	 *
	 * @param   boolean			If true, the view output will be cached
	 * @param   array  An array of safe url parameters and their variable types, for valid values see {@link JFilterInput::clean()}.
	 *
	 * @return  \JController		This object to support chaining.
	 * @since   1.5
	 */
	public function display($cachable = false, $urlparams = false)
	{
		$results = $this->dispatcher->trigger( 'onConferenceStartPage', array($this->container));
		$result = $this->responseHandler->handle($results);
		$this->_render(array('content' => $result));

	}

	/**
	 * Method to process entering conference room
	 */
	public function enter()
	{
		$results = $this->dispatcher->trigger( 'onConferenceEnterRoom', array($this->container));
		$result = $this->responseHandler->handle($results);
		$this->_render(array('content' => $result));
	}

	/**
	 * This method implemented to leave ability to create custom events in plugins
	 * If 'some_custom_event' will be called as task then 'onConferenceSomeCustomEvent' will be triggered
	 * Result from plugin is gonna be rendered as HTML content
	 *
	 * @param string $name
	 * @param string $arguments
	 */
	public function custom()
	{
		$eventName = $this->input->get('task', '');
		$index = strpos($eventName, '_');
		if($index !== false){
			$subStrToReplace = substr($eventName, $index, 2);
			$eventName = str_replace($subStrToReplace, strtoupper($subStrToReplace[1]), $eventName);
		}
		$eventName[0] = strtoupper($eventName[0]);

		$results = $this->dispatcher->trigger( 'onConference'.$eventName, array($this->container));
		$result = $this->responseHandler->handle($results);
		$this->_render(array('content' => $result));
	}


	protected function _render($data = array())
	{
		$view = $this->getView($this->input->get('view'), $this->input->get('format', 'html'));
		foreach($data as $key=>$value){
			$view->$key = $value;
		}
		$view->display($this->input->get('layout'));
	}

	/**
	 * Initialize parameters for plugin according to input data, plugin params and settings
	 * TODO: break this method to small ones
	 *
	 * @throws \Exception
	 */
	private function _init()
	{
		$registry = ServiceLocator::get('conference_registry');
		$paramsCB = new \JLMSParameters($this->config->get('course_params'));
		$enableBooking = (bool)$paramsCB->get('conf_book', 0);

		$courseID = $this->input->get('id', 0);
		$courseID = $this->input->get('course_id', $courseID);

		if($courseID == 0){
			throw new \Exception('Course is not selected');
		}

		$params = $_REQUEST;
		$params['isManager'] = $this->acl->CheckPermissions('conference', 'manage');
		$params['enableBooking'] = $enableBooking;
		$params['courseID'] = $courseID;
		$params['ItemID'] = $this->input->get('Itemid', 0);
		$params['my'] = $this->user;
		$params['usertype'] = $params['isManager'] ? 1 : 2;
		$registry->set('params', $params);

		$this->container['db'] = ServiceLocator::get('db');
		$this->container['config'] = $this->config;
		$this->container['session'] = \JLMSFactory::getSession();
		$this->container['registry'] = $registry;
		$this->container['dateHandler'] = ServiceLocator::get('dateHandler');
		$this->container['router'] = ServiceLocator::get('conference_router');
		$this->container['acl'] = $this->acl;
		$this->container['editor'] = ServiceLocator::get('conference_editor');

		$options = array(
			'cachebase' => $this->config->get('tmp_path')
		);
		$this->container->setContext('cacheDriver', $options);

		jimport('joomla.cache.storage.file');
		$this->container['cacheDriver'] = create_function('$options', 'return new JCacheStorageFile($options);');
	}

}