<?php
namespace LMS\Tools\Coursebuilder\Repositories;

use LMS\Libraries\Base;
use LMS\Tools\Coursebuilder\Tables;

class ModulesMenu extends Base\Repository
{
	public function __construct($options = array())
	{
		parent::__construct($options);
		$this->table = new Tables\ModulesMenu($this->db);
	}

	/*
	 * Use standard joomla driver DB
	 */
	public function save($data)
	{
		$this->table->bind($data);
		$this->table->check();
		$this->table->store();
		return $this->table->moduleid;
	}

	protected function prepareFilter($query, &$filterOptions)
	{
		$alias = $this->table->getTableAlias();

		$conditions = array();

		if(isset($filterOptions['id']) && $filterOptions['id']){
			$conditions[] = $alias.".".$this->table->getKeyName()." = ".$this->db->quote($filterOptions['id']);
		}

		if(isset($filterOptions['moduleid']) && $filterOptions['moduleid']){
			$conditions[] = $alias.".moduleid = ".$this->db->quote($filterOptions['moduleid']);
		}

		if(isset($filterOptions['menuid']) && $filterOptions['menuid']){
			$conditions[] = $alias.".menuid = ".$this->db->quote($filterOptions['menuid']);
		}

		$query->where($conditions);

		return parent::prepareFilter($query, $filterOptions); // TODO: Change the autogenerated stub
	}

	public function getModuleIdsByMenuItemid($itemId)
	{
		return $this->getByFilter(
			array(
				'fields' => $this->getTable()->getTableAlias().'.moduleid',
				'menuid' => $itemId,
			)
		);
	}

	public function getModulesByMenuItemid($itemId)
	{
		return $this->getByFilter(
			array(
				'fields' => $this->getTable()->getTableAlias().'.moduleid, '.$this->getTable()->getTableAlias().'.modules.*',
				'menuid' => $itemId,
			)
		);
	}

	public function getModuleIdsByModuleId($moduleId)
	{
		return $this->getByFilter(
			array(
				'fields' => $this->getTable()->getTableAlias().'.*',
				'moduleid' => $moduleId,
			)
		);
	}

	public function getRecordsByModuleidAndItemid($moduleId, $itemId)
	{
		return $this->getByFilter(
			array(
				'moduleid' => $moduleId,
				'menuid' => $itemId,
			)
		);
	}


}